import sys
import subprocess
import os
import platform
import json
import shutil
from PyQt5 import QtWidgets, QtGui, QtCore

# === CONSTANTS ===
APP_NAME = "SoftEther_Tray_Data"
APP_VERSION = "V1.5"
DATA_DIR = os.path.join(os.path.dirname(os.path.abspath(__file__)), APP_NAME)
CONFIG_FILE = os.path.join(DATA_DIR, "config.json")
BATCH_FILE = os.path.join(DATA_DIR, "start_softether_monitor.bat")

DEFAULT_REFRESH_INTERVAL = 10000  # 10 seconds

# === Ensure data directory exists ===
os.makedirs(DATA_DIR, exist_ok=True)

# === CONFIGURATION ===
def load_config():
    print("Loading configuration...")
    if os.path.exists(CONFIG_FILE):
        try:
            with open(CONFIG_FILE, "r") as file:
                config = json.load(file)
                print("Configuration loaded successfully.")
                return config
        except Exception as e:
            print(f"Error loading config file: {e}")
            return prompt_for_config()
    else:
        print("No configuration file found.")
        return prompt_for_config()

def prompt_for_config():
    app = QtWidgets.QApplication(sys.argv)
    dialog = InitialSetupDialog()
    if dialog.exec_() == QtWidgets.QDialog.Accepted:
        config = dialog.get_config()
        save_config(config)
        return config
    else:
        print("Setup cancelled.")
        sys.exit(0)

def save_config(config):
    print("Saving configuration...")
    try:
        with open(CONFIG_FILE, "w") as file:
            json.dump(config, file, indent=4)
        print("Configuration saved.")
    except Exception as e:
        print(f"Error saving config: {e}")

# === MAIN WIDGET ===
class VPNWidget(QtWidgets.QWidget):
    def __init__(self):
        super().__init__()
        self.setWindowTitle("SoftEther VPN Sessions")
        self.setGeometry(1000, 700, 500, 300)
        self.setWindowFlags(QtCore.Qt.Tool | QtCore.Qt.WindowStaysOnTopHint)

        layout = QtWidgets.QVBoxLayout()
        self.text = QtWidgets.QTextEdit()
        self.text.setReadOnly(True)
        layout.addWidget(self.text)
        self.setLayout(layout)

    def update_text(self, new_text):
        self.text.setPlainText(new_text)

# === VPN FETCHER THREAD ===
class VPNFetcher(QtCore.QThread):
    session_data_ready = QtCore.pyqtSignal(str, list)

    def __init__(self, hub_name, password, vpncmd_path):
        super().__init__()
        self.hub_name = hub_name
        self.password = password
        self.vpncmd_path = vpncmd_path

    def run(self):
        try:
            startupinfo = None
            creationflags = 0
            if platform.system() == "Windows":
                startupinfo = subprocess.STARTUPINFO()
                startupinfo.dwFlags |= subprocess.STARTF_USESHOWWINDOW
                startupinfo.wShowWindow = subprocess.SW_HIDE
                creationflags = subprocess.CREATE_NO_WINDOW

            command = [
                self.vpncmd_path,
                "localhost",
                "/SERVER",
                f"/PASSWORD:{self.password}"
            ]

            process = subprocess.Popen(
                command,
                stdin=subprocess.PIPE,
                stdout=subprocess.PIPE,
                stderr=subprocess.PIPE,
                universal_newlines=True,
                shell=False,
                startupinfo=startupinfo,
                creationflags=creationflags
            )

            input_commands = f"\nHub {self.hub_name}\nSessionList\nexit\n"
            output, error = process.communicate(input=input_commands, timeout=30)

            if process.returncode != 0:
                self.session_data_ready.emit(f"[ERROR] {error}", [])
                return

            lines = output.splitlines()
            sessions = []
            session = {}
            capture_keys = ["Session Name", "User Name", "Source Host Name"]

            for line in lines:
                if "|" in line:
                    key, value = map(str.strip, line.split("|", 1))
                    if key == "Session Name" and session:
                        sessions.append(session)
                        session = {}
                    if key in capture_keys:
                        session[key] = value

            if session:
                sessions.append(session)

            if not sessions:
                display = "No active sessions."
                session_ids = []
            else:
                display = ""
                session_ids = []
                for i, s in enumerate(sessions, 1):
                    display += f"Session {i}:\n"
                    display += f"  Session Name : {s.get('Session Name', 'N/A')}\n"
                    display += f"  User Name    : {s.get('User Name', 'N/A')}\n"
                    display += f"  Source IP    : {s.get('Source Host Name', 'N/A')}\n\n"
                    session_ids.append(s.get("Session Name", ""))

            self.session_data_ready.emit(display.strip(), session_ids)

        except Exception as e:
            self.session_data_ready.emit(f"[ERROR] {str(e)}", [])

# === SYSTEM TRAY APP ===
class TrayApp(QtWidgets.QSystemTrayIcon):
    def __init__(self, icon, config, parent=None):
        super().__init__(icon, parent)
        self.setToolTip("SoftEther Tray Monitor")
        self.config = config
        self.widget = VPNWidget()
        self.previous_sessions = set()

        menu = QtWidgets.QMenu()
        menu.addAction("Show/Hide", self.toggle_widget)
        menu.addAction("Settings", self.open_settings)
        menu.addAction("Startup Setup", self.setup_startup)
        menu.addAction("About", self.show_about)  # <-- Added About
        menu.addAction("Exit", QtWidgets.qApp.quit)
        self.setContextMenu(menu)

        self.activated.connect(self.on_click)

        self.timer = QtCore.QTimer()
        self.timer.timeout.connect(self.fetch_sessions)
        self.timer.start(self.config["refresh_interval"])

        self.show()
        self.fetch_sessions()

    def on_click(self, reason):
        if reason == QtWidgets.QSystemTrayIcon.Trigger:
            self.toggle_widget()

    def toggle_widget(self):
        if self.widget.isVisible():
            self.widget.hide()
        else:
            self.widget.show()

    def fetch_sessions(self):
        self.worker = VPNFetcher(self.config["hub_name"], self.config["password"], self.config["vpncmd_path"])
        self.worker.session_data_ready.connect(self.handle_session_update)
        self.worker.start()

    def handle_session_update(self, display_text, session_names):
        self.widget.update_text(display_text)
        current_sessions = set(session_names)
        new_sessions = current_sessions - self.previous_sessions

        if new_sessions:
            for session in new_sessions:
                self.showMessage("New VPN Session", f"New session: {session}", QtWidgets.QSystemTrayIcon.Information)

        self.previous_sessions = current_sessions

    def open_settings(self):
        dialog = SettingsDialog(self.config)
        if dialog.exec_():
            self.config = dialog.get_config()
            save_config(self.config)
            self.timer.start(self.config["refresh_interval"])
            self.fetch_sessions()

    def setup_startup(self):
        try:
            import win32com.client
        except ImportError:
            reply = QtWidgets.QMessageBox.question(None, "Missing Dependency", "pywin32 is required for startup setup. Install it now?", QtWidgets.QMessageBox.Yes | QtWidgets.QMessageBox.No)
            if reply == QtWidgets.QMessageBox.Yes:
                subprocess.check_call([sys.executable, "-m", "pip", "install", "pywin32"])
            else:
                return

        pythonw = shutil.which("pythonw.exe")
        if not pythonw:
            QtWidgets.QMessageBox.critical(None, "Error", "pythonw.exe not found in PATH.")
            return

        script_path = os.path.abspath(__file__)
        with open(BATCH_FILE, "w") as f:
            f.write(f"@echo off\nstart \"\" \"{pythonw}\" \"{script_path}\"\nexit\n")

        startup_dir = os.path.join(os.getenv('APPDATA'), "Microsoft", "Windows", "Start Menu", "Programs", "Startup")
        shortcut_path = os.path.join(startup_dir, "SoftEtherMonitor.lnk")

        shell = win32com.client.Dispatch("WScript.Shell")
        shortcut = shell.CreateShortcut(shortcut_path)
        shortcut.TargetPath = BATCH_FILE
        shortcut.WorkingDirectory = DATA_DIR
        shortcut.Save()

        QtWidgets.QMessageBox.information(None, "Startup Setup", "Startup shortcut created successfully!")

    def show_about(self):
        QtWidgets.QMessageBox.information(
            None,
            "About",
            f"SoftEther Tray Monitor\n"
            f"{APP_VERSION}\n"
            f"Created by ChatGPT & James Cooke\n"
            f"www.jamescooke.me.uk"
        )

# === SETTINGS DIALOG ===
class SettingsDialog(QtWidgets.QDialog):
    def __init__(self, config, parent=None):
        super().__init__(parent)
        self.setWindowTitle("Settings")
        self.setFixedSize(350, 250)

        self.config = config

        layout = QtWidgets.QFormLayout(self)

        self.hub_name_edit = QtWidgets.QLineEdit(self.config["hub_name"])
        self.password_edit = QtWidgets.QLineEdit(self.config["password"])
        self.password_edit.setEchoMode(QtWidgets.QLineEdit.Password)
        self.refresh_interval_edit = QtWidgets.QSpinBox()
        self.refresh_interval_edit.setValue(self.config["refresh_interval"] // 1000)
        self.refresh_interval_edit.setRange(1, 60)
        self.vpncmd_path_edit = QtWidgets.QLineEdit(self.config["vpncmd_path"])

        layout.addRow("Hub Name:", self.hub_name_edit)
        layout.addRow("Password:", self.password_edit)
        layout.addRow("Refresh Interval (s):", self.refresh_interval_edit)
        layout.addRow("vpncmd Path:", self.vpncmd_path_edit)

        buttons = QtWidgets.QDialogButtonBox(QtWidgets.QDialogButtonBox.Ok | QtWidgets.QDialogButtonBox.Cancel)
        buttons.accepted.connect(self.accept)
        buttons.rejected.connect(self.reject)
        layout.addWidget(buttons)

    def get_config(self):
        return {
            "hub_name": self.hub_name_edit.text(),
            "password": self.password_edit.text(),
            "refresh_interval": self.refresh_interval_edit.value() * 1000,
            "vpncmd_path": self.vpncmd_path_edit.text()
        }

# === INITIAL SETUP DIALOG ===
class InitialSetupDialog(QtWidgets.QDialog):
    def __init__(self, parent=None):
        super().__init__(parent)
        self.setWindowTitle("Initial Setup")
        self.setFixedSize(350, 250)

        layout = QtWidgets.QFormLayout(self)

        self.hub_name_edit = QtWidgets.QLineEdit()
        self.password_edit = QtWidgets.QLineEdit()
        self.password_edit.setEchoMode(QtWidgets.QLineEdit.Password)
        self.refresh_interval_edit = QtWidgets.QSpinBox()
        self.refresh_interval_edit.setValue(DEFAULT_REFRESH_INTERVAL // 1000)
        self.refresh_interval_edit.setRange(1, 60)

        default_path = self.detect_default_vpncmd()
        self.vpncmd_path_edit = QtWidgets.QLineEdit(default_path)

        layout.addRow("Hub Name:", self.hub_name_edit)
        layout.addRow("Password:", self.password_edit)
        layout.addRow("Refresh Interval (s):", self.refresh_interval_edit)
        layout.addRow("vpncmd Path:", self.vpncmd_path_edit)

        buttons = QtWidgets.QDialogButtonBox(QtWidgets.QDialogButtonBox.Ok | QtWidgets.QDialogButtonBox.Cancel)
        buttons.accepted.connect(self.accept)
        buttons.rejected.connect(self.reject)
        layout.addWidget(buttons)

    def detect_default_vpncmd(self):
        possible_path = r"C:\\Program Files\\SoftEther VPN Server\\vpncmd_x64.exe"
        return possible_path if os.path.exists(possible_path) else ""

    def get_config(self):
        return {
            "hub_name": self.hub_name_edit.text(),
            "password": self.password_edit.text(),
            "refresh_interval": self.refresh_interval_edit.value() * 1000,
            "vpncmd_path": self.vpncmd_path_edit.text()
        }

# === MAIN FUNCTION ===
def main():
    print("Starting the application...")
    config = load_config()

    app = QtWidgets.QApplication(sys.argv)
    app.setQuitOnLastWindowClosed(False)

    if not QtWidgets.QSystemTrayIcon.isSystemTrayAvailable():
        print("System tray not available.")
        return

    icon_path = os.path.join(os.path.dirname(os.path.abspath(__file__)), "icon.png")
    icon = QtGui.QIcon(icon_path)

    tray = TrayApp(icon, config)

    sys.exit(app.exec_())

if __name__ == "__main__":
    main()
